<?php
/**
 * @package     Joomla.Administrator
 * @copyright   Copyright (C) 2005 - 2012 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

//J4 for using StringHelper:: instead of JString::
Use \Joomla\String\StringHelper;

jimport('joomla.application.component.modellist');
require_once( JPATH_ADMINISTRATOR.'/components/com_ipricecalc/helpers/discount.php');	

class IpricecalcModelPrice extends JModelList
{
	public $_context = 'com_ipricecalc.price';
	protected $_extension = 'com_ipricecalc';
	private $_parent = null;
	private $_items = null;

	protected function getStoreId($id = '')
	{
		$id	 = ':'.$this->getState('filter.published');
		$id	.= ':'.$this->getState('filter.search');
		$id	.= ':'.$this->getState('filter.catids');
		return parent::getStoreId($id);
	}

	protected function getListQuery()
	{
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		
		$query->select(
			$this->getState(
				'list.select',
				'ipi.*'
			)
		);
		$query->from($db->quoteName('#__iprice_item').' AS ipi');

		$query->select('ipc.name AS category');
		$query->join('', $db->quoteName('#__iprice_categories').' AS ipc ON ipc.id = ipi.catid AND ipc.published = 1');
		
		$query->where('ipi.published = 1');
		
		$catids = $this->getState('filter.catids');
		if ($catids)
			$query->where('ipi.catid in ('.$catids.')');
			
		$wh = $this->getState('filter.where');
		if ($wh)
			$query->where($wh);
			
		$search = $this->getState('filter.search');
		if (!empty($search)) {
			$search = $db->Quote('%'.$db->escape($search, true).'%');
			$query->where('(ipi.name LIKE '.$search.' OR ipi.shortdesc LIKE '.$search.')');
		}
		$orderCol	= $this->state->get('list.ordering', '');
		if (!$orderCol)
			$orderCol = 'ipc.ordering, ipi.ordering';
		else 
			$orderCol = 'ipc.ordering,' . $orderCol;
		$orderDirn	= $this->state->get('list.direction', 'ASC');
		$query->order($db->escape($orderCol.' '.$orderDirn));
		
		
		return $query;
	}
	
	protected function populateState($ordering = 'ordering', $direction = 'ASC')
	{
		$app = JFactory::getApplication();

		$this->setState('list.limit', 0);
		$this->setState('list.start', 0);

		//$orderCol = JString::substr($app->input->getVar('filter_order', 'ipi.ordering'),0,50);
		//J4
		$orderCol = StringHelper::substr($app->input->getVar('filter_order', 'ipi.ordering'),0,50);
		
		$this->setState('list.ordering', $orderCol);

		$listOrder = $app->input->getVar('filter_order_Dir', 'ASC');
		if (!in_array(strtoupper($listOrder), array('ASC', 'DESC', ''))) {
			$listOrder = 'ASC';
		}
		$this->setState('list.direction', $listOrder);
	}

	public function getStart()
	{
		return $this->getState('list.start');
	}
	
	function getNextOrderNo()
	{
		$db = JFactory::getDBO();
		$query = 'SELECT MAX(orderno) from #__iprice_order_main';
		$db->setQuery($query);
		$nextorder = 1 + (int) $db->LoadResult();
		return $nextorder;
	}
	
	function updateTotals($orderno,$currcode = 'RUR', $pparams = null)
	{
		
		if (!$pparams)
			$pparams = JFactory::getApplication()->getParams('com_ipricecalc');
		$db = JFactory::getDBO();
		$query = 'update #__iprice_order as ipo set amount = (select cost from #__iprice_item as ipi where ipi.id=ipo.itemid ) where orderno = '. (int) $orderno;
		$db->setQuery($query);
		if (!$db->execute())
			JError::raiseError( 500, $db->stderr());
			
		$curr = JTable::getInstance('curr', 'IpricecalcTable');
		$curr = $curr->getCurr($currcode);
		$decimal = (int) $pparams->get('decimal','2');
		
		$roundrate=1;
		for ($i=1;$i<=$decimal;$i++)
			$roundrate = $roundrate*10;
		
		if ($curr->rate && $curr->rate!=1) 
		{
			if ($decimal)
				$amnt = 'round(amount * '.$curr->rate.'*'.$roundrate.')/'.$roundrate;
			else
				$amnt = 'round(amount * '.$curr->rate.')';
			$query = 'update #__iprice_order as ipo set amount = '.$amnt.' where orderno = '. (int) $orderno;
			$db->setQuery($query);
			if (!$db->execute())
				JError::raiseError( 500, $db->stderr());
		}
		
		$query = 'select sum(amount*countitems) from #__iprice_order as ipo where ipo.orderno='.$db->Quote($orderno);
		$db->setQuery($query);
		$total = $db->LoadResult();
		$grandtotal = $total;
		
		$discs = IPDiscountHelper::getSortDiscount($pparams);
		$disc = 0;
		foreach ($discs as $key=>$value)
		{
			if ($total >= $value) {
				$disc = $key;
				break;
			}
		}
		if ($disc !=0)
			$grandtotal = $total - $total/100*$disc;
		
		$query = 'select totalratio from #__iprice_order_main as ipom where ipom.orderno='.$db->Quote($orderno);
		$db->setQuery($query);
		$ordertotalratio = $db->LoadResult();
		if ($ordertotalratio && $ordertotalratio>0) 
			$grandtotal = $grandtotal * $ordertotalratio;
		
		$query = 'update #__iprice_order_main as ipom set grandtotal ='. (float)$grandtotal. ', totalamount ='. (float)$total. ', discratio = '.(int)$disc.' where ipom.orderno='.(int)$orderno;
		
		$db->setQuery($query);
		if (!$db->execute())
			JError::raiseError( 500, $db->stderr());
	}
	
	function insOrder( $items, $contact, $orderno,$currcode = 'RUR', & $pparams )
	{
		//$msg = 'Count items '.count($items);
		if (!$contact)
		  return false;
	  
		$db = JFactory::getDBO();

		$query = 'INSERT INTO #__iprice_order_main ( orderno, orderdate, fio, firm, email, phone, comments , totalamount, currcode, totalratio, totalratioview , showtotalratio)' .  
					' VALUES ( '.(int) $orderno.', NOW() , '.$db->Quote($contact->fio).', '.$db->Quote($contact->firm).', '.$db->Quote($contact->email).', '.$db->Quote($contact->phone).', '.$db->Quote($contact->comments).', 0 ,'.$db->Quote($currcode).', '.(float) $pparams->get('totalratio','0').', '.(int) $pparams->get('totalratioview','0').', '.(int) $pparams->get('showtotalratio','1').')';
		
		$db->setQuery($query);
		if (!$db->execute())
			JError::raiseError( 500, $db->stderr());
					
		foreach ($items as $item) {
			if ($item->cnt) {
				$query = 'INSERT INTO #__iprice_order ( orderno, itemid, countitems, amount, field1, field2, field3, field4, field5)' .
						' select '.(int) $orderno.', '.(int) $item->id.', '.(float) $item->cnt.', cost, field1, field2, field3, field4, field5' .
						' from #__iprice_item where id='.(int) $item->id;
				$db->setQuery($query);
				if (!$db->execute())
					JError::raiseError( 500, $db->stderr());
			}
		}
		$this->updateTotals($orderno, $currcode, $pparams);
		$query = 'SELECT id from #__iprice_order_main where orderno='.$db->Quote($orderno);
		$db->setQuery($query);
		$orid = $db->LoadResult();
		return $orid;
	}
	
}
?>
