<?php

/**
 * @package		Joomla
 * @copyright	Copyright (C) 2013 IPrice web solutions. All rights reserved.
 * @license		GNU/GPL, see LICENSE.php
 */

defined ( '_JEXEC' ) or die ();


class IpriceSpamRecaptcha {
	protected $error = '';
	protected $publickey = null;
	protected $privatekey = null;
	protected $host = null;

	public function __construct() {
		$mainframe = JFactory::getApplication();			
		$pparams = $mainframe->getPageParameters('com_ipricecalc');		
		$this->publickey = $pparams->get('captchapublickey','');
		$this->privatekey = $pparams->get('captchaprivatekey','');
		$this->host = JFactory::getApplication()->input->getVar('REMOTE_ADDR', null, 'server');
	
	}

	public static function getInstance() {
		static $instance = null;
		if (!$instance) $instance = new IpriceSpamRecaptcha();
		return $instance;
	}


	public function getHtml () {
		if ( empty($this->publickey) ) {
			$this->error = JText::sprintf ( 'IPRICE_RECAPTCHA_ERROR_INVALID_CONFIGURATION', JText::_ ( 'IPRICE_RECAPTCHA_ERROR_INVALID_CONFIGURATION_NO_PUBLIC_KEY' ) );
			return false;
		}
		if ( empty($this->privatekey) ) {
			$this->error = JText::sprintf ( 'IPRICE_RECAPTCHA_ERROR_INVALID_CONFIGURATION', JText::_ ( 'IPRICE_RECAPTCHA_ERROR_INVALID_CONFIGURATION_NO_PRIVATE_KEY' ) );
			return false;
		}

		JFactory::getDocument()->addCustomTag('<script type="text/javascript">
		<!--
			var RecaptchaOptions = {
				custom_translations : {
				instructions_visual : "'.JText::_('IPRICE_RECAPTCHA_JS_INSTRUCTIONS_VISUAL').'",
				instructions_audio : "'.JText::_('IPRICE_RECAPTCHA_JS_INSTRUCTIONS_AUDIO').'",
				play_again : "'.JText::_('IPRICE_RECAPTCHA_JS_PLAY_AGAIN').'",
				cant_hear_this : "'.JText::_('IPRICE_RECAPTCHA_JS_CANT_HEAR_THIS').'",
				visual_challenge : "'.JText::_('IPRICE_RECAPTCHA_JS_VISUAL_CHALLENGE').'",
				audio_challenge : "'.JText::_('IPRICE_RECAPTCHA_JS_AUDIO_CHALLENGE').'",
				refresh_btn : "'.JText::_('IPRICE_RECAPTCHA_JS_REFRESH_CODE').'",
				help_btn : "'.JText::_('IPRICE_RECAPTCHA_JS_HELP_BTN').'",
				incorrect_try_again : "'.JText::_('IPRICE_RECAPTCHA_ERROR_INCORRECT_CAPTCHA_SOL').'",
			},
			theme : "white"
			};
		//-->
		</script>
		');

		$server = 'http'.($this->_isSSLConnection() ? 's' : '').'://www.google.com/recaptcha/api';

		$errorpart = '';
		/*
		if ($this->error) {
			$errorpart = "&error=" . urlencode($this->error);
		}
		*/

		return '<script type="text/javascript" src="'. $server . '/challenge?k=' . $this->publickey . $errorpart . '"></script>
		<noscript>
			<iframe src="'. $server . '/noscript?k=' . $this->publickey . $errorpart . '" height="300" width="500" frameborder="0"></iframe><br />
			<textarea name="recaptcha_challenge_field" rows="3" cols="40"></textarea>
			<input type="hidden" name="recaptcha_response_field" value="manual_challenge" />
		</noscript>';
	}


	public function verify($extra_params = array()) {
		if ( empty($this->privatekey) ) {
			$this->error =  JText::sprintf ( 'IPRICE_RECAPTCHA_ERROR_INVALID_CONFIGURATION', JText::_ ( 'IPRICE_RECAPTCHA_ERROR_INVALID_CONFIGURATION_NO_PRIVATE_KEY' ) );
			return false;
		}

		$challenge	= JFactory::getApplication()->input->getString('recaptcha_challenge_field');
		$response	= JFactory::getApplication()->input->getString('recaptcha_response_field');

		// Discard spam
		if ($challenge == null || strlen($challenge) == 0 || $response == null || strlen($response) == 0) {
			$this->error = JText::_ ( 'IPRICE_RECAPTCHA_ERROR_INCORRECT_CAPTCHA_SOL' );
			return false;
		}

		$response = $this->_query ("/recaptcha/api/verify",
			array (
			'privatekey' => $this->privatekey,
			'remoteip' => $this->host,
			'challenge' => $challenge,
			'response' => $response
			) + $extra_params
		);

		if (!$response) {
			$response[1] = "false\nrecaptcha-not-reachable";
		}
		$answers = preg_split('/[\s,]+/', $response[1]);
		if (empty($answers[0]) || trim ($answers [0]) != 'true') {
			$this->error = JText::_ ( 'IPRICE_RECAPTCHA_ERROR_' . preg_replace('/[^\w\d]+/', '_', strtoupper($answers[1])) );
			return false;
		}
		return true;
	}

	/**
	 * Get error string
	 * @return string Error code
	 */
	public function getError() {
		return $this->error;
	}

	protected function _isSSLConnection()
	{
		return ((isset($_SERVER['HTTPS']) &&
			($_SERVER['HTTPS'] == 'on')) ||
			getenv('SSL_PROTOCOL_VERSION'));
	}

	/**
	* Encodes the given data into a query string format
	* @param $data - array of string elements to be encoded
	* @return string - encoded request
	*/
	private function _encode ($data) {
		$req = '';
		foreach ( $data as $key => $value )
			$req .= $key . '=' . urlencode( $value ) . '&';

		// Cut the last '&'
		$req=substr($req,0,strlen($req)-1);
		return $req;
	}

	/**
	* Submits an HTTP POST to a reCAPTCHA server
	* @param string $path
	* @param array $data
	* @param int port
	* @return array response
	*/
	private function _query($path, $data, $port = 80) {
		$req = $this->_encode ($data);
		$host = 'www.google.com';
		$http_request  = "POST $path HTTP/1.0\r\n";
		$http_request .= "Host: $host\r\n";
		$http_request .= "Content-Type: application/x-www-form-urlencoded;\r\n";
		$http_request .= "Content-Length: " . strlen($req) . "\r\n";
		$http_request .= "User-Agent: IPrice\r\n";
		$http_request .= "\r\n";
		$http_request .= $req;

		$response = '';
		if( false == ( $fs = @fsockopen($host, $port, $errno, $errstr, 10) ) ) {
			return null;
		}

		fwrite($fs, $http_request);

		while ( !feof($fs) )
			$response .= fgets($fs, 1160); // One TCP-IP packet
		fclose($fs);
		$response = explode("\r\n\r\n", $response, 2);

		return $response;
	}
}
?>